"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MongoMemoryReplSet = exports.MongoMemoryReplSetEvents = exports.MongoMemoryReplSetStates = void 0;
const tslib_1 = require("tslib");
const events_1 = require("events");
const MongoMemoryServer_1 = require("./MongoMemoryServer");
const utils_1 = require("./util/utils");
const debug_1 = tslib_1.__importDefault(require("debug"));
const mongodb_1 = require("mongodb");
const MongoInstance_1 = require("./util/MongoInstance");
const errors_1 = require("./util/errors");
const log = debug_1.default('MongoMS:MongoMemoryReplSet');
/**
 * Enum for "_state" inside "MongoMemoryReplSet"
 */
var MongoMemoryReplSetStates;
(function (MongoMemoryReplSetStates) {
    MongoMemoryReplSetStates["init"] = "init";
    MongoMemoryReplSetStates["running"] = "running";
    MongoMemoryReplSetStates["stopped"] = "stopped";
})(MongoMemoryReplSetStates = exports.MongoMemoryReplSetStates || (exports.MongoMemoryReplSetStates = {}));
/**
 * All Events for "MongoMemoryReplSet"
 */
var MongoMemoryReplSetEvents;
(function (MongoMemoryReplSetEvents) {
    MongoMemoryReplSetEvents["stateChange"] = "stateChange";
})(MongoMemoryReplSetEvents = exports.MongoMemoryReplSetEvents || (exports.MongoMemoryReplSetEvents = {}));
/**
 * Class for managing an replSet
 */
class MongoMemoryReplSet extends events_1.EventEmitter {
    constructor(opts = {}) {
        var _a;
        super();
        /**
         * All servers this ReplSet instance manages
         */
        this.servers = [];
        this._state = MongoMemoryReplSetStates.stopped;
        this._ranCreateAuth = false;
        this.binaryOpts = Object.assign({}, opts.binary);
        this.instanceOpts = (_a = opts.instanceOpts) !== null && _a !== void 0 ? _a : [];
        this.replSetOpts = Object.assign({}, opts.replSet);
    }
    /**
     * Change "this._state" to "newState" and emit "newState"
     * @param newState The new State to set & emit
     */
    stateChange(newState, ...args) {
        this._state = newState;
        this.emit(MongoMemoryReplSetEvents.stateChange, newState, ...args);
    }
    /**
     * Create an instance of "MongoMemoryReplSet" and call start
     * @param opts Options for the ReplSet
     */
    static create(opts) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            log('create: Called .create() method');
            const replSet = new this(Object.assign({}, opts));
            yield replSet.start();
            return replSet;
        });
    }
    /**
     * Get Current state of this class
     */
    get state() {
        return this._state;
    }
    /**
     * Get & Set "instanceOpts"
     * @throws if "state" is not "stopped"
     */
    get instanceOpts() {
        return this._instanceOpts;
    }
    set instanceOpts(val) {
        assertionIsMMSRSState(MongoMemoryReplSetStates.stopped, this._state);
        this._instanceOpts = val;
    }
    /**
     * Get & Set "binaryOpts"
     * @throws if "state" is not "stopped"
     */
    get binaryOpts() {
        return this._binaryOpts;
    }
    set binaryOpts(val) {
        assertionIsMMSRSState(MongoMemoryReplSetStates.stopped, this._state);
        this._binaryOpts = val;
    }
    /**
     * Get & Set "replSetOpts"
     * (Applies defaults)
     * @throws if "state" is not "stopped"
     */
    get replSetOpts() {
        return this._replSetOpts;
    }
    set replSetOpts(val) {
        assertionIsMMSRSState(MongoMemoryReplSetStates.stopped, this._state);
        const defaults = {
            auth: false,
            args: [],
            name: 'testset',
            count: 1,
            dbName: utils_1.generateDbName(),
            ip: '127.0.0.1',
            spawn: {},
            storageEngine: 'ephemeralForTest',
            configSettings: {},
        };
        this._replSetOpts = Object.assign(Object.assign({}, defaults), val);
        utils_1.assertion(this._replSetOpts.count > 0, new Error('ReplSet Count needs to be 1 or higher!'));
        if (typeof this._replSetOpts.auth === 'object') {
            this._replSetOpts.auth = utils_1.authDefault(this._replSetOpts.auth);
        }
    }
    /**
     * Returns instance options suitable for a MongoMemoryServer.
     * @param baseOpts Options to merge with
     */
    getInstanceOpts(baseOpts = {}) {
        const opts = {
            // disable "auth" if replsetopts has an object-auth
            auth: typeof this._replSetOpts.auth === 'object' && !this._ranCreateAuth
                ? false
                : !!this._replSetOpts.auth,
            args: this._replSetOpts.args,
            dbName: this._replSetOpts.dbName,
            ip: this._replSetOpts.ip,
            replSet: this._replSetOpts.name,
            storageEngine: this._replSetOpts.storageEngine,
        };
        if (baseOpts.args) {
            opts.args = this._replSetOpts.args.concat(baseOpts.args);
        }
        if (baseOpts.port) {
            opts.port = baseOpts.port;
        }
        if (baseOpts.dbPath) {
            opts.dbPath = baseOpts.dbPath;
        }
        if (baseOpts.storageEngine) {
            opts.storageEngine = baseOpts.storageEngine;
        }
        if (baseOpts.replicaMemberConfig) {
            opts.replicaMemberConfig = baseOpts.replicaMemberConfig;
        }
        log('getInstanceOpts: instance opts:', opts);
        return opts;
    }
    /**
     * Returns an mongodb URI that is setup with all replSet servers
     * @param otherDb add an database into the uri (in mongodb its the auth database, in mongoose its the default database for models)
     * @throws if state is not "running"
     * @throws if an server doesnt have "instanceInfo.port" defined
     * @return an valid mongo URI, by the definition of https://docs.mongodb.com/manual/reference/connection-string/
     */
    getUri(otherDb) {
        log('getUri:', this.state);
        switch (this.state) {
            case MongoMemoryReplSetStates.running:
            case MongoMemoryReplSetStates.init:
                break;
            case MongoMemoryReplSetStates.stopped:
            default:
                throw new errors_1.StateError([MongoMemoryReplSetStates.running, MongoMemoryReplSetStates.init], this.state);
        }
        const hosts = this.servers
            .map((s) => {
            var _a;
            const port = (_a = s.instanceInfo) === null || _a === void 0 ? void 0 : _a.port;
            utils_1.assertion(!utils_1.isNullOrUndefined(port), new Error('Instance Port is undefined!'));
            return `127.0.0.1:${port}`;
        })
            .join(',');
        return utils_1.uriTemplate(hosts, undefined, utils_1.generateDbName(otherDb), [
            `replicaSet=${this._replSetOpts.name}`,
        ]);
    }
    /**
     * Start underlying `mongod` instances.
     * @throws if state is already "running"
     */
    start() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            log('start:', this.state);
            switch (this.state) {
                case MongoMemoryReplSetStates.stopped:
                    break;
                case MongoMemoryReplSetStates.running:
                default:
                    throw new errors_1.StateError([MongoMemoryReplSetStates.stopped], this.state);
            }
            this.stateChange(MongoMemoryReplSetStates.init); // this needs to be executed before "setImmediate"
            // check if an "beforeExit" listener for "this.cleanup" is already defined for this class, if not add one
            if (process
                .listeners('beforeExit')
                .findIndex((f) => f === this.cleanup) <= -1) {
                process.on('beforeExit', this.cleanup);
            }
            yield utils_1.ensureAsync()
                .then(() => this.initAllServers())
                .then(() => this._initReplSet())
                .catch((err) => {
                if (!debug_1.default.enabled('MongoMS:MongoMemoryReplSet')) {
                    console.warn('Starting the ReplSet failed, enable debug for more information');
                }
                this.stateChange(MongoMemoryReplSetStates.stopped);
                throw err;
            });
        });
    }
    /**
     * Initialize & start all servers in the replSet
     */
    initAllServers() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            log('initAllServers');
            this.stateChange(MongoMemoryReplSetStates.init);
            if (this.servers.length > 0) {
                log('initAllServers: lenght of "servers" is higher than 0, starting existing servers');
                yield Promise.all(this.servers.map((s) => s.start(true)));
                return;
            }
            // Any servers defined within `_instanceOpts` should be started first as
            // the user could have specified a `dbPath` in which case we would want to perform
            // the `replSetInitiate` command against that server.
            this._instanceOpts.forEach((opts, index) => {
                log(`initAllServers: starting special server "${index + 1}" of "${this._instanceOpts.length}" from instanceOpts (count: ${this.servers.length + 1}):`, opts);
                this.servers.push(this._initServer(this.getInstanceOpts(opts)));
            });
            while (this.servers.length < this._replSetOpts.count) {
                log(`initAllServers: starting extra server "${this.servers.length + 1}" of "${this._replSetOpts.count}" (count: ${this.servers.length + 1})`);
                this.servers.push(this._initServer(this.getInstanceOpts()));
            }
            log('initAllServers: waiting for all servers to finish starting');
            // ensures all servers are listening for connection
            yield Promise.all(this.servers.map((s) => s.start()));
        });
    }
    /**
     * Stop the underlying `mongod` instance(s).
     * @param runCleanup run "this.cleanup"? (remove dbPath & reset "instanceInfo")
     */
    stop(runCleanup = true) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            log('stop' + utils_1.isNullOrUndefined(process.exitCode) ? '' : ': called by process-event');
            if (this._state === MongoMemoryReplSetStates.stopped) {
                return false;
            }
            const bool = yield Promise.all(this.servers.map((s) => s.stop(false)))
                .then(() => {
                this.stateChange(MongoMemoryReplSetStates.stopped);
                return true;
            })
                .catch((err) => {
                log('stop:', err);
                this.stateChange(MongoMemoryReplSetStates.stopped, err);
                return false;
            });
            // return early if the instances failed to stop
            if (!bool) {
                return bool;
            }
            if (runCleanup) {
                yield this.cleanup(false);
            }
            return true;
        });
    }
    /**
     * Remove the defined dbPath's
     * This function gets automatically called on process event "beforeExit" (with force being "false")
     * @param force Remove the dbPath even if it is no "tmpDir" (and re-check if tmpDir actually removed it)
     * @throws If "state" is not "stopped"
     * @throws If "instanceInfo" is not defined
     * @throws If an fs error occured
     */
    cleanup(force = false) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            assertionIsMMSRSState(MongoMemoryReplSetStates.stopped, this._state);
            log(`cleanup for "${this.servers.length}" servers`);
            process.removeListener('beforeExit', this.cleanup);
            yield Promise.all(this.servers.map((s) => s.cleanup(force)));
            this.servers = [];
            return;
        });
    }
    /**
     * Wait until all instances are running
     * @throws if state is "stopped" (cannot wait on something that dosnt start)
     */
    waitUntilRunning() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield utils_1.ensureAsync();
            log('waitUntilRunning:', this._state);
            switch (this._state) {
                case MongoMemoryReplSetStates.running:
                    // just return immediatly if the replSet is already running
                    return;
                case MongoMemoryReplSetStates.init:
                    // wait for event "running"
                    yield new Promise((res) => {
                        // the use of "this" here can be done because "on" either binds "this" or uses an arrow function
                        function waitRunning(state) {
                            // this is because other states can be emitted multiple times (like stopped & init for auth creation)
                            if (state === MongoMemoryReplSetStates.running) {
                                this.removeListener(MongoMemoryReplSetEvents.stateChange, waitRunning);
                                res();
                            }
                        }
                        this.on(MongoMemoryReplSetEvents.stateChange, waitRunning);
                    });
                    return;
                case MongoMemoryReplSetStates.stopped:
                default:
                    throw new errors_1.StateError([MongoMemoryReplSetStates.running, MongoMemoryReplSetStates.init], this.state);
            }
        });
    }
    /**
     * Connects to the first server from the list of servers and issues the `replSetInitiate`
     * command passing in a new replica set configuration object.
     * @throws if state is not "init"
     * @throws if "servers.length" is not 1 or above
     * @throws if package "mongodb" is not installed
     */
    _initReplSet() {
        var _a;
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            log('_initReplSet');
            assertionIsMMSRSState(MongoMemoryReplSetStates.init, this._state);
            utils_1.assertion(this.servers.length > 0, new Error('One or more servers are required.'));
            const uris = this.servers.map((server) => server.getUri());
            let con = yield mongodb_1.MongoClient.connect(uris[0], {
                useNewUrlParser: true,
                useUnifiedTopology: true,
            });
            log('_initReplSet: connected');
            // try-finally to close connection in any case
            try {
                let adminDb = con.db('admin');
                const members = uris.map((uri, index) => {
                    var _a;
                    return (Object.assign({ _id: index, host: utils_1.getHost(uri) }, (((_a = this.servers[index].opts.instance) === null || _a === void 0 ? void 0 : _a.replicaMemberConfig) || {})));
                });
                const rsConfig = {
                    _id: this._replSetOpts.name,
                    members,
                    settings: Object.assign({ electionTimeoutMillis: 500 }, this._replSetOpts.configSettings),
                };
                // try-catch because the first "command" can fail
                try {
                    log('_initReplSet: trying "replSetInitiate"');
                    yield adminDb.command({ replSetInitiate: rsConfig });
                    if (typeof this._replSetOpts.auth === 'object') {
                        log('_initReplSet: "this._replSetOpts.auth" is an object');
                        yield this._waitForPrimary();
                        const primary = this.servers.find((server) => { var _a; return (_a = server.instanceInfo) === null || _a === void 0 ? void 0 : _a.instance.isInstancePrimary; });
                        utils_1.assertion(!utils_1.isNullOrUndefined(primary), new Error('No Primary found'));
                        utils_1.assertion(!utils_1.isNullOrUndefined(primary.instanceInfo), new Error('Primary dosnt have an "instanceInfo" defined'));
                        yield primary.createAuth(primary.instanceInfo);
                        this._ranCreateAuth = true;
                        if (((_a = primary.opts.instance) === null || _a === void 0 ? void 0 : _a.storageEngine) !== 'ephemeralForTest') {
                            log('_initReplSet: closing connection for restart');
                            yield con.close(); // close connection in preparation for "stop"
                            yield this.stop(false); // stop all servers for enabling auth
                            log('_initReplSet: starting all server again with auth');
                            yield this.initAllServers(); // start all servers again with "auth" enabled
                            con = yield mongodb_1.MongoClient.connect(this.getUri('admin'), {
                                useNewUrlParser: true,
                                useUnifiedTopology: true,
                                authSource: 'admin',
                                authMechanism: 'SCRAM-SHA-256',
                                auth: {
                                    user: this._replSetOpts.auth.customRootName,
                                    password: this._replSetOpts.auth.customRootPwd,
                                },
                            });
                            adminDb = con.db('admin');
                            log('_initReplSet: auth restart finished');
                        }
                        else {
                            console.warn('Not Restarting ReplSet for Auth\n' +
                                'Storage engine of current PRIMARY is ephemeralForTest, which does not write data on shutdown, and mongodb does not allow changing "auth" runtime');
                        }
                    }
                }
                catch (e) {
                    if (e instanceof mongodb_1.MongoError && e.errmsg == 'already initialized') {
                        log(`_initReplSet: "${e.errmsg}": trying to set old config`);
                        const { config: oldConfig } = yield adminDb.command({ replSetGetConfig: 1 });
                        log('_initReplSet: got old config:\n', oldConfig);
                        yield adminDb.command({
                            replSetReconfig: oldConfig,
                            force: true,
                        });
                    }
                    else {
                        throw e;
                    }
                }
                log('_initReplSet: ReplSet-reconfig finished');
                yield this._waitForPrimary();
                this.stateChange(MongoMemoryReplSetStates.running);
                log('_initReplSet: running');
            }
            finally {
                yield con.close();
            }
        });
    }
    /**
     * Create the one Instance (without starting them)
     * @param instanceOpts Instance Options to use for this instance
     */
    _initServer(instanceOpts) {
        const serverOpts = {
            binary: this._binaryOpts,
            instance: instanceOpts,
            spawn: this._replSetOpts.spawn,
            auth: typeof this.replSetOpts.auth === 'object' ? this.replSetOpts.auth : undefined,
        };
        const server = new MongoMemoryServer_1.MongoMemoryServer(serverOpts);
        return server;
    }
    /**
     * Wait until the replSet has elected an Primary
     * @param timeout Timeout to not run infinitly
     * @throws if timeout is reached
     */
    _waitForPrimary(timeout = 30000) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            log('_waitForPrimary: Waiting for an Primary');
            let timeoutId;
            // "race" because not all servers will be an primary
            yield Promise.race([
                ...this.servers.map((server) => new Promise((res, rej) => {
                    const instanceInfo = server.instanceInfo;
                    if (utils_1.isNullOrUndefined(instanceInfo)) {
                        return rej(new Error('_waitForPrimary - instanceInfo not present'));
                    }
                    instanceInfo.instance.once(MongoInstance_1.MongoInstanceEvents.instancePrimary, res);
                    if (instanceInfo.instance.isInstancePrimary) {
                        log('_waitForPrimary: found instance being already primary');
                        res();
                    }
                })),
                new Promise((_res, rej) => {
                    timeoutId = setTimeout(() => {
                        rej(new Error(`Timed out after ${timeout}ms while waiting for an Primary`));
                    }, timeout);
                }),
            ]);
            if (!utils_1.isNullOrUndefined(timeoutId)) {
                clearTimeout(timeoutId);
            }
            log('_waitForPrimary: detected one primary instance ');
        });
    }
}
exports.MongoMemoryReplSet = MongoMemoryReplSet;
exports.default = MongoMemoryReplSet;
/**
 * Helper function to de-duplicate state checking for "MongoMemoryReplSetStates"
 * @param wantedState The State that is wanted
 * @param currentState The current State ("this._state")
 */
function assertionIsMMSRSState(wantedState, currentState) {
    utils_1.assertion(currentState === wantedState, new errors_1.StateError([wantedState], currentState));
}
//# sourceMappingURL=MongoMemoryReplSet.js.map